package cn.newfeifan.mall.module.product.controller.admin.spuapply;

import cn.newfeifan.mall.module.product.convert.spuapply.SpuApplyConvert;
import cn.newfeifan.mall.module.product.dal.dataobject.skuapply.SkuApplyDO;
import cn.newfeifan.mall.module.product.dal.mysql.spuapply.SpuApplyMapper;
import cn.newfeifan.mall.module.product.service.skuapply.SkuApplyService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.newfeifan.mall.framework.common.pojo.PageParam;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.pojo.CommonResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;
import static cn.newfeifan.mall.framework.common.pojo.CommonResult.success;

import cn.newfeifan.mall.framework.excel.core.util.ExcelUtils;

import cn.newfeifan.mall.framework.operatelog.core.annotations.OperateLog;
import static cn.newfeifan.mall.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.newfeifan.mall.module.product.controller.admin.spuapply.vo.*;
import cn.newfeifan.mall.module.product.dal.dataobject.spuapply.SpuApplyDO;
import cn.newfeifan.mall.module.product.service.spuapply.SpuApplyService;

@Tag(name = "管理后台 - 商品spu申请")
@RestController
@RequestMapping("/product/spu-apply")
@Validated
public class SpuApplyController {

    @Resource
    private SpuApplyService spuApplyService;
    @Resource
    private SpuApplyMapper spuApplyMapper;
    @Resource
    private SkuApplyService skuApplyService;

    @PostMapping("/create")
    @Operation(summary = "创建商品spu申请")
    @PreAuthorize("@ss.hasPermission('product:spu-apply:create')")
    public CommonResult<Long> createSpuApply(@Valid @RequestBody SpuApplySaveReqVO createReqVO) {
        return success(spuApplyService.createSpuApply(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新商品spu申请")
    @PreAuthorize("@ss.hasPermission('product:spu-apply:update')")
    public CommonResult<Boolean> updateSpuApply(@Valid @RequestBody SpuApplySaveReqVO updateReqVO) {
        spuApplyService.updateSpuApply(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除商品spu申请")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('product:spu-apply:delete')")
    public CommonResult<Boolean> deleteSpuApply(@RequestParam("id") Long id) {
        spuApplyService.deleteSpuApply(id);
        return success(true);
    }

    @GetMapping("/get-detail")
    @Operation(summary = "获得商品 spu 申请 明细")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('product:spu-apply:query')")
    public CommonResult<SpuApplyRespVO> getSpuApply(@RequestParam("id") Long id) {
        // 获得商品 SPU 申请
        SpuApplyDO spu = spuApplyService.getSpu(id);
        if (spu == null) {
            return success(null);
        }
        // 查询商品 SKU
        List<SkuApplyDO> skus = skuApplyService.getSkuListBySpuId(spu.getId());

        String shopName = spuApplyMapper.selectShopNameById(spu.getShopId());
        CommonResult<SpuApplyRespVO> success = success(SpuApplyConvert.INSTANCE.convert(spu, skus));
        success.getData().setShopName(shopName);

        return success;
    }

    @GetMapping("/page")
    @Operation(summary = "获得商品spu申请分页")
    @PreAuthorize("@ss.hasPermission('product:spu-apply:query')")
    public CommonResult<PageResult<SpuApplyRespVO>> getSpuApplyPage(@Valid SpuApplyPageReqVO pageReqVO) {
        PageResult<SpuApplyDO> pageResult = spuApplyService.getSpuApplyPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, SpuApplyRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出商品spu申请 Excel")
    @PreAuthorize("@ss.hasPermission('product:spu-apply:export')")
    @OperateLog(type = EXPORT)
    public void exportSpuApplyExcel(@Valid SpuApplyPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<SpuApplyDO> list = spuApplyService.getSpuApplyPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "商品spu申请.xls", "数据", SpuApplyRespVO.class,
                        BeanUtils.toBean(list, SpuApplyRespVO.class));
    }

}